/*
 * Copyright (C) 2003 ichitokumei, tt. 
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#include <stdlib.h>

#include <qfile.h>
#include <qdir.h>
#include <qlayout.h>
#include <qdialog.h>
#include <qlabel.h>
#include <qtoolbutton.h>
#include <qtabwidget.h>
#include <qspinbox.h>
#include <qpopupmenu.h>
#include <qaction.h>
#include <qmessagebox.h>
#include <qapplication.h>
#include <qpe/qpetoolbar.h>
#include <qpe/qpemenubar.h>
#include <qpe/resource.h>
#include "inkwp.h"
#include "inkwpmemo.h"

#define DIRNAME (getenv("HOME") + QString("/Applications/inkwp/"))

QString inkwp_datafile(int page) {
  return DIRNAME + QString::number(page) + ".inkwp";
}

InkWp::InkWp(QWidget *parent, const char *name) 
  : QMainWindow(parent, name) {
  setToolBarsMovable(false);
  QPEToolBar *toolBar = new QPEToolBar(this);
  toolBar->setHorizontalStretchable(true);
  QPEMenuBar *menuBar = new QPEMenuBar(toolBar);

  QPopupMenu *menuMenu = new QPopupMenu(this);
  menuBar->insertItem("Menu", menuMenu);

  QAction *a;

  a = new QAction("New Page", "New Page", 0, this);
  a->addTo(menuMenu);
  connect(a, SIGNAL(activated()), this, SLOT(slotNewPage()));
  a = new QAction("Del Page", "Del Page", 0, this);
  a->addTo(menuMenu);
  connect(a, SIGNAL(activated()), this, SLOT(slotDeletePage()));
  a = new QAction("Clear Page", "Clear Page", 0, this);
  a->addTo(menuMenu);
  connect(a, SIGNAL(activated()), this, SLOT(slotClear()));

  menuMenu->insertSeparator();

  QPopupMenu *menuSize = new QPopupMenu(this);
  menuMenu->insertItem("Size", menuSize);

  QActionGroup *ag = new QActionGroup(this);
  a = new QAction("12", "12", 0, ag);
  a->setToggleAction(true);
  a = new QAction("16", "16", 0, ag);
  a->setToggleAction(true);
  a->setOn(true);
  a = new QAction("24", "24", 0, ag);
  a->setToggleAction(true);
  a = new QAction("32", "32", 0, ag);
  a->setToggleAction(true);
  a = new QAction("48", "48", 0, ag);
  a->setToggleAction(true);

  ag->addTo(menuSize);
  ag->setExclusive(true);
  ag->setEnabled(true);
  connect(ag, SIGNAL(selected(QAction *)), 
		  this, SLOT(slotSizeSelected(QAction *)));

  a = new QAction("Export PNG", "Export PNG", 0, this);
  a->addTo(menuMenu);
  connect(a, SIGNAL(activated()), this, SLOT(slotPng()));
  a = new QAction("Memo", "Memo", 0, this);
  a->addTo(menuMenu);
  connect(a, SIGNAL(activated()), this, SLOT(slotMemo()));


  QPEToolBar *buttonBar = new QPEToolBar(this);


  toolButtonPoint = new QToolButton(buttonBar);
  toolButtonPoint->setPixmap(Resource::loadPixmap("inkwp/point"));
  toolButtonPoint->setToggleButton(true);
  toolButtonTwo = new QToolButton(buttonBar);
  toolButtonTwo->setPixmap(Resource::loadPixmap("inkwp/two"));
  toolButtonTwo->setToggleButton(true);

  buttonBar->addSeparator();

  toolButtonSpace = new QToolButton(buttonBar);
  toolButtonSpace->setPixmap(Resource::loadPixmap("inkwp/space"));
  toolButtonBS = new QToolButton(buttonBar);
  toolButtonBS->setPixmap(Resource::loadPixmap("inkwp/backspace"));
  toolButtonDeleteLine = new QToolButton(buttonBar);
  toolButtonDeleteLine->setPixmap(Resource::loadPixmap("inkwp/delline"));
  toolButtonEnter = new QToolButton(buttonBar);
  toolButtonEnter->setPixmap(Resource::loadPixmap("inkwp/enter"));
									

  inkWpData = new InkWpData;
  load(0);
  tabWidget = new QTabWidget(this, 0);
  setCentralWidget(tabWidget);
  tabSize = 0;
  inkWpWidgetList.setAutoDelete(TRUE);
  for (;;) {
    if (tabSize == 0 || QFile::exists(inkwp_datafile(tabSize))) {
      slotNewPage();
    } else {
      break;
    }
  }
  tabWidget->setCurrentPage(0);
  inkWpWidget = static_cast<InkWpWidget*>(tabWidget->currentPage());
  inkWpWidget->setFocus();

  connect(toolButtonPoint, SIGNAL(clicked()), this, SLOT(slotPoint()));
  connect(tabWidget, SIGNAL(currentChanged(QWidget*)), this, SLOT(slotTabChanged(QWidget*)));
}
void InkWp::closeEvent(QCloseEvent *e) {
  save(inkWpWidget->page());
  e->accept();
}

void InkWp::slotPoint() {
  toolButtonPoint->setEnabled(false);
  inkWpWidget->selectPoint();
}
void InkWp::slotPointSelected() {
  toolButtonPoint->setOn(false);
  toolButtonPoint->setEnabled(true);
}

void InkWp::slotSizeSelected(QAction *a) {
  setCharHeight(inkWpWidget, a->text().toInt());
}

void InkWp::slotTabChanged(QWidget *w) {
  save(inkWpWidget->page());
  inkWpWidget = static_cast<InkWpWidget*>(w);
  load(inkWpWidget->page());
  inkWpWidget->jumpTo(QPoint(0, 0));
}

void InkWp::slotNewPage() {
  InkWpWidget *w = insertNewPage(tabSize);
  tabWidget->showPage(w);
}

InkWpWidget* InkWp::insertNewPage(int page) {
  InkWpWidget *w = new InkWpWidget(tabWidget, 0, inkWpData);
  w->setPage(page);
  setCharHeight(w, 16);
  connect(toolButtonSpace, SIGNAL(clicked()), w, SLOT(slotSpace()));
  connect(toolButtonBS, SIGNAL(clicked()), w, SLOT(slotBS()));
  connect(toolButtonDeleteLine, SIGNAL(clicked()), w, SLOT(slotDeleteLine()));
  connect(toolButtonEnter, SIGNAL(clicked()), w, SLOT(slotEnter()));
  connect(toolButtonTwo, SIGNAL(toggled(bool)), w, SLOT(setTwoLineMode(bool)));
  connect(w, SIGNAL(pointSelected()), this, SLOT(slotPointSelected()));
  tabWidget->insertTab(w, QString::number(page));
  inkWpWidgetList.append(w);
  tabSize++;
  return w;
}

void InkWp::slotDeletePage() {
  if (tabSize == 1)
    return;

  QMessageBox msg("Delete Page", "Are you sure?", QMessageBox::Warning, 
				  QMessageBox::Ok, QMessageBox::Cancel, QMessageBox::NoButton, 
				  this);
  if (msg.exec() == QMessageBox::Cancel)
    return;

  InkWpWidget* w = inkWpWidget;
  int page = w->page();

  // delete a page
  tabWidget->removePage(w);
  inkWpWidgetList.removeRef(w);

  QDir qd;
  if (page == tabSize-1) {
    // remove the last file
    qd.remove(inkwp_datafile(page));
  } else {
    // shift files
    for (int i=page+1; i<tabSize; i++) {
      qd.rename(inkwp_datafile(i), inkwp_datafile(i-1));
    }
    // assign the new page number
    QListIterator<InkWpWidget> itr(inkWpWidgetList);
    for (page=0; itr.current(); ++itr) {
      InkWpWidget *x = itr.current();
      x->setPage(page);
      tabWidget->changeTab(x, QString::number(page));
      page++;
    }
  }
  tabSize = page;
}

void InkWp::slotMemo() {
  InkWpMemo dlg(this, 0, inkWpWidget->page(), inkWpData, inkWpWidget->charHeight());
  dlg.showMaximized();
  dlg.exec();
}
void InkWp::slotPng() {
  QDialog dlg(this, 0, true);
  QSpinBox *sbWidth, *sbCharHeight;
  QGridLayout *l = new QGridLayout(&dlg, 2, 2);
  QLabel *label;
  label = new QLabel("width", &dlg);
  l->addWidget(label, 0, 0);
  label = new QLabel("char height", &dlg);
  l->addWidget(label, 0, 1);
  sbWidth = new QSpinBox(100, 1000, 1, &dlg);
  sbWidth->setValue(inkWpWidget->width());
  l->addWidget(sbWidth, 1, 0);
  sbCharHeight = new QSpinBox(12, 48, 1, &dlg);
  l->addWidget(sbCharHeight, 1, 1);

  if (dlg.exec() == QDialog::Accepted) {
    QFile f(getenv("HOME") + QString("/Documents/Image_Files/inkwp.png"));
    f.open(IO_WriteOnly);
    QDataStream ds(&f);
    int i = 
      inkWpData->numberOfLines(sbWidth->value(), sbCharHeight->value(), 
		    sbCharHeight->value()/4, sbCharHeight->value()/4);
    QPixmap pm(sbWidth->value(), 
	       sbCharHeight->value() * i + (sbCharHeight->value()/4) * i);
    pm.fill(QColor(255, 255, 255));
    QPainter painter(&pm);
    inkWpData->draw(painter, 0, sbWidth->value(), sbCharHeight->value(), 
		    sbCharHeight->value()/4, sbCharHeight->value()/4);

    ds << pm;
    f.close();
  }
}
void InkWp::slotClear() {
  QMessageBox msg("Clear Page", "Are you sure?", QMessageBox::Warning, 
				  QMessageBox::Ok, QMessageBox::Cancel, QMessageBox::NoButton, 
				  this);
  if (msg.exec() == QMessageBox::Ok) {
	inkWpData->clear();
	save(inkWpWidget->page());
	inkWpWidget->jumpTo(QPoint(0, 0));
  }
}

void InkWp::setCharHeight(InkWpWidget* w, int h) {
  w->setDimensions(h, h/4, h/8);
}

void InkWp::save(int page) {
  QDir qd;
  if (!qd.exists(DIRNAME))
    qd.mkdir(DIRNAME);
  inkWpData->save(inkwp_datafile(page));
}
void InkWp::load(int page) {
  QDir qd;
  if (!qd.exists(DIRNAME))
    qd.mkdir(DIRNAME);
  inkWpData->clear();
  inkWpData->load(inkwp_datafile(page));
}
