/*
 * Copyright (C) 2003 ichitokumei, tt. 
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef INKWPDATA_H
#define INKWPDATA_H

#include <qarray.h>
#include <qbitarray.h>
#include <qvaluelist.h>
#include <qpainter.h>
#include <qpoint.h>

class InkWpChar {
public:
  enum Type {Handwriting, String};

  InkWpChar();
  InkWpChar(Type); 
  InkWpChar(int); 
  InkWpChar(QString); 

  void setType(Type); 
  Type type() const; 

  void setStandardHeight(int);
  int standardHeight() const;

  int width(int h);

  int draw(QPainter &, int x, int baseLine, int h);

  void addPoint(QPoint, bool _connected);

  void setData(const QPointArray &, const QBitArray &);
  void setText(const QString &);
  const QPointArray &points() const {return points_;}
  const QBitArray &connected() const {return connected_;}

  const QString &text() const {return text_;}
private:
  Type type_;

  int standardHeight_;
  QPointArray points_;
  QBitArray connected_;

  QString text_;
};

QDataStream &operator<< (QDataStream &, const InkWpChar &);
QDataStream &operator>> (QDataStream &, InkWpChar &);

// returen values: true if successfull
class InkWpData {
 public:
  InkWpData();
  bool save(QString); 
  bool load(QString);
  void clear();
  const InkWpChar &at(int);
  int count() const;
  bool remove(int);
  bool insert(int, const InkWpChar &); // -1 => append

  int pointToIndex(QPoint, 
		   int width, int charHeight, 
		   int charMargin, int rowMargin); 
  QPoint draw(QPainter &, int currentIndex, 
		   int width, int charHeight, 
		   int charMargin, int rowMargin); 
  // call this only after draw.
  int numberOfLines(int width, int charHeight, 
		    int charMargin, int rowMargin);

 private:
  QValueList<InkWpChar> data_;
};

#endif // INKWPDATA_H
