/*
 * Copyright (C) 2003 ichitokumei.
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#include "inkwpmemo.h"
#include "inkwpwidget.h"
#include <qtextstream.h>
#include <qfile.h>
#include <qmultilineedit.h>
#include <qlayout.h>
#include <stdlib.h>

class ScrolledInkWpWidget : public QScrollView {
public:
  ScrolledInkWpWidget(QWidget *parent, const char *name, 
		      InkWpData *inkWpData, int charHeight) 
  : QScrollView(parent, name), data(inkWpData), h(charHeight) {
    w = new InkWpWidget(this, 0, data, true);
    w->setDimensions(h, h/4, h/4);
    addChild(w);
    setHScrollBarMode(QScrollView::AlwaysOff);
    setVScrollBarMode(QScrollView::AlwaysOn);
  }
  void viewportResizeEvent(QResizeEvent *) {
    int i = data->numberOfLines(viewport()->width(), h, h/4, h/4);
    w->setFixedSize(viewport()->width(), h*i + (h/4)*i);
  }
private:
  InkWpData *data;
  InkWpWidget *w;
  int h;
};

InkWpMemo::InkWpMemo(QWidget* parent, const char* name, int i, 
		     InkWpData* d, int charHeight) :
  QDialog(parent, name, true),  currentPage(i), 
  data_(d)
{
  QVBoxLayout *l = new QVBoxLayout(this);
  mle = new QMultiLineEdit(this);
  l->addWidget(mle);
  ScrolledInkWpWidget *sv = new ScrolledInkWpWidget(this, 0, d, charHeight);
  l->addWidget(sv);

  QFile f(getenv("HOME") + QString("/Documents/Text_Files/inkwp_") 
	  + QString::number(currentPage) + ".txt");
  if (f.open(IO_ReadOnly)) {
    QTextStream ts(&f);

    while(!ts.eof()) {
      mle->insertLine(ts.readLine());
    }
    f.close();
  }
}

InkWpMemo::~InkWpMemo() {
  QFile f(getenv("HOME") + QString("/Documents/Text_Files/inkwp_") 
	  + QString::number(currentPage) + ".txt");
  f.open(IO_WriteOnly);
  QTextStream ts(&f);

  ts << mle->text();

  f.close();
}
