/*
 * Copyright (C) 2003 ichitokumei, tt. 
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef INKWPWIDGET_H
#define INKWPWIDGET_H

#include <qwidget.h>
#include <qpointarray.h>
#include <qbitarray.h>

class InkWpData;
class QPainter;
class QTimer;

class InkWpWidget : public QWidget
{ 
  Q_OBJECT

 public:
  InkWpWidget(QWidget* parent, const char* name, InkWpData *data, 
	      bool readOnly = false);
  bool dirty() {return dirty_;}
  void jumpTo(QPoint);
  void selectPoint() {waitForPoint_ = true;}
  int  page() const {return currentPage_;}
  void setPage(int p) {currentPage_ = p;}
  int  charHeight() {return charHeight_;}

 public slots:
  void slotSpace(); 
  void slotBS();
  void slotDeleteLine();
  void slotEnter();
  void setTwoLineMode(bool);
  void setDimensions(int charHeight, int charMargin, int rowMargin);

 protected:
  void mousePressEvent(QMouseEvent*);
  void mouseReleaseEvent(QMouseEvent*);
  void mouseMoveEvent(QMouseEvent*);
  void keyPressEvent(QKeyEvent*);
  void paintEvent(QPaintEvent*);

 private:
  InkWpData *data_;
  int countLines();

  int currentPage_;
  int currentIndex_;
  bool dirty_, readOnly_;
  bool waitForPoint_, twoLineMode_;
  int charHeight_, charMargin_, rowMargin_;

  QPoint p0, p1;
  QPointArray points;
  QBitArray connected;

  QTimer *timer;

  void load(QString);
  void save(QString);

  QPoint currentPos_;

  int lastX, lastY;

 signals:
  void pointSelected();

 private slots:
  void slotTimeOut();

};
#endif // INKWPWIDGET_H
